@@ -26,7 +26,6 @@
 
 #include "pangocairo.h"
 #include "pangocairo-private.h"
-#include "pango-font-private.h"
 #include "pango-impl-utils.h"
 
 #define PANGO_CAIRO_FONT_PRIVATE(font)		\
@@ -93,7 +92,7 @@ done:
     {
       cairo_scaled_font_t *scaled_font = cf_priv->scaled_font;
       PangoFont *font = PANGO_FONT (cf_priv->cfont);
-      static GQuark warned_quark = 0; /* MT-safe */
+      static GQuark warned_quark = 0;
       if (!warned_quark)
 	warned_quark = g_quark_from_static_string ("pangocairo-scaledfont-warned");
 
@@ -143,7 +142,7 @@ done:
  * The scaled font can be referenced and kept using
  * cairo_scaled_font_reference().
  *
- * Return value: (transfer none) (nullable): the #cairo_scaled_font_t used by @font,
+ * Return value: the #cairo_scaled_font_t used by @font,
  *               or %NULL if @font is %NULL.
  *
  * Since: 1.18
@@ -224,7 +223,6 @@ _pango_cairo_font_get_metrics (PangoFont     *font,
   PangoCairoFontPrivate *cf_priv = PANGO_CAIRO_FONT_PRIVATE (font);
   PangoCairoFontMetricsInfo *info = NULL; /* Quiet gcc */
   GSList *tmp_list;
-  static int in_get_metrics;
 
   const char *sample_str = pango_language_get_sample_string (language);
 
@@ -251,7 +249,6 @@ _pango_cairo_font_get_metrics (PangoFont     *font,
       cairo_matrix_t cairo_matrix;
       PangoMatrix pango_matrix;
       PangoMatrix identity = PANGO_MATRIX_INIT;
-      glong sample_str_width;
 
       int height, shift;
 
@@ -294,43 +291,33 @@ _pango_cairo_font_get_metrics (PangoFont     *font,
 
 	  info->metrics->ascent *= xscale;
 	  info->metrics->descent *= xscale;
-	  info->metrics->height *= xscale;
 	  info->metrics->underline_position *= xscale;
 	  info->metrics->underline_thickness *= xscale;
 	  info->metrics->strikethrough_position *= xscale;
 	  info->metrics->strikethrough_thickness *= xscale;
 	}
 
+
       /* Set the matrix on the context so we don't have to adjust the derived
        * metrics. */
       pango_context_set_matrix (context, &pango_matrix);
 
-      /* Ugly. We need to prevent recursion when we call into
-       * PangoLayout to determine approximate char width.
-       */
-      if (!in_get_metrics)
-        {
-          in_get_metrics = 1;
+      /* Update approximate_*_width now */
+      layout = pango_layout_new (context);
+      desc = pango_font_describe_with_absolute_size (font);
+      pango_layout_set_font_description (layout, desc);
+      pango_font_description_free (desc);
 
-          /* Update approximate_*_width now */
-          layout = pango_layout_new (context);
-          desc = pango_font_describe_with_absolute_size (font);
-          pango_layout_set_font_description (layout, desc);
-          pango_font_description_free (desc);
+      pango_layout_set_text (layout, sample_str, -1);
+      pango_layout_get_extents (layout, NULL, &extents);
 
-          pango_layout_set_text (layout, sample_str, -1);
-          pango_layout_get_extents (layout, NULL, &extents);
+      info->metrics->approximate_char_width = extents.width / pango_utf8_strwidth (sample_str);
 
-          sample_str_width = pango_utf8_strwidth (sample_str);
-          g_assert (sample_str_width > 0);
-          info->metrics->approximate_char_width = extents.width / sample_str_width;
+      pango_layout_set_text (layout, "0123456789", -1);
+      info->metrics->approximate_digit_width = max_glyph_width (layout);
 
-          pango_layout_set_text (layout, "0123456789", -1);
-          info->metrics->approximate_digit_width = max_glyph_width (layout);
+      g_object_unref (layout);
 
-          g_object_unref (layout);
-          in_get_metrics = 0;
-        }
 
       /* We may actually reuse ascent/descent we got from cairo here.  that's
        * in cf_priv->font_extents.
@@ -370,7 +357,7 @@ _pango_cairo_font_get_metrics (PangoFont     *font,
 static PangoCairoFontHexBoxInfo *
 _pango_cairo_font_private_get_hex_box_info (PangoCairoFontPrivate *cf_priv)
 {
-  const char hexdigits[] = "0123456789ABCDEF";
+  static const char hexdigits[] = "0123456789ABCDEF";
   char c[2] = {0, 0};
   PangoFont *mini_font;
   PangoCairoFontHexBoxInfo *hbi;
@@ -389,8 +376,8 @@ _pango_cairo_font_private_get_hex_box_info (PangoCairoFontPrivate *cf_priv)
   double size, mini_size;
   PangoFontDescription *desc;
   cairo_scaled_font_t *scaled_font, *scaled_mini_font;
-  PangoMatrix pango_ctm, pango_font_matrix;
-  cairo_matrix_t cairo_ctm, cairo_font_matrix;
+  PangoMatrix pango_ctm;
+  cairo_matrix_t cairo_ctm;
   /*PangoGravity gravity;*/
 
   if (!cf_priv)
@@ -407,10 +394,10 @@ _pango_cairo_font_private_get_hex_box_info (PangoCairoFontPrivate *cf_priv)
 
   font_options = cairo_font_options_create ();
   desc = pango_font_describe_with_absolute_size ((PangoFont *)cf_priv->cfont);
+  size = pango_font_description_get_size (desc) / (1.*PANGO_SCALE);
   /*gravity = pango_font_description_get_gravity (desc);*/
 
   cairo_scaled_font_get_ctm (scaled_font, &cairo_ctm);
-  cairo_scaled_font_get_font_matrix (scaled_font, &cairo_font_matrix);
   cairo_scaled_font_get_font_options (scaled_font, font_options);
   /* I started adding support for vertical hexboxes here, but it's too much
    * work.  Easier to do with cairo user fonts and vertical writing mode
@@ -423,15 +410,8 @@ _pango_cairo_font_private_get_hex_box_info (PangoCairoFontPrivate *cf_priv)
   pango_ctm.yy = cairo_ctm.yy;
   pango_ctm.x0 = cairo_ctm.x0;
   pango_ctm.y0 = cairo_ctm.y0;
-  pango_font_matrix.xx = cairo_font_matrix.xx;
-  pango_font_matrix.yx = cairo_font_matrix.yx;
-  pango_font_matrix.xy = cairo_font_matrix.xy;
-  pango_font_matrix.yy = cairo_font_matrix.yy;
-  pango_font_matrix.x0 = cairo_font_matrix.x0;
-  pango_font_matrix.y0 = cairo_font_matrix.y0;
 
-  size = pango_matrix_get_font_scale_factor (&pango_font_matrix) /
-	 pango_matrix_get_font_scale_factor (&pango_ctm);
+  size /= pango_matrix_get_font_scale_factor (&pango_ctm);
 
   if (is_hinted)
     {
@@ -508,8 +488,6 @@ _pango_cairo_font_private_get_hex_box_info (PangoCairoFontPrivate *cf_priv)
 
 
   scaled_mini_font = pango_cairo_font_get_scaled_font ((PangoCairoFont *) mini_font);
-  if (G_UNLIKELY (scaled_mini_font == NULL || cairo_scaled_font_status (scaled_mini_font) != CAIRO_STATUS_SUCCESS))
-    return NULL;
 
   for (i = 0 ; i < 16 ; i++)
     {
@@ -671,45 +649,6 @@ _pango_cairo_font_private_is_metrics_hinted (PangoCairoFontPrivate *cf_priv)
   return cf_priv->is_hinted;
 }
 
-static void
-get_space_extents (PangoCairoFontPrivate *cf_priv,
-                   PangoRectangle        *ink_rect,
-                   PangoRectangle        *logical_rect)
-{
-  const char hexdigits[] = "0123456789ABCDEF";
-  char c[2] = {0, 0};
-  int i;
-  double hex_width;
-  int width;
-
-  /* we don't render missing spaces as hex boxes,
-   * so come up with some width to use. For lack
-   * of anything better, use average hex digit width.
-   */
-
-  hex_width = 0;
-  for (i = 0 ; i < 16 ; i++)
-    {
-      cairo_text_extents_t extents;
-
-      c[0] = hexdigits[i];
-      cairo_scaled_font_text_extents (cf_priv->scaled_font, c, &extents);
-      hex_width += extents.width;
-    }
-  width = pango_units_from_double (hex_width / 16);
-
-  if (ink_rect)
-    {
-      ink_rect->x = ink_rect->y = ink_rect->height = 0;
-      ink_rect->width = width;
-    }
-  if (logical_rect)
-    {
-      *logical_rect = cf_priv->font_extents;
-      logical_rect->width = width;
-    }
-}
-
 static void
 _pango_cairo_font_private_get_glyph_extents_missing (PangoCairoFontPrivate *cf_priv,
 						     PangoGlyph             glyph,
@@ -720,14 +659,6 @@ _pango_cairo_font_private_get_glyph_extents_missing (PangoCairoFontPrivate *cf_p
   gunichar ch;
   gint rows, cols;
 
-  ch = glyph & ~PANGO_GLYPH_UNKNOWN_FLAG;
-
-  if (ch == 0x20 || ch == 0x2423)
-    {
-      get_space_extents (cf_priv, ink_rect, logical_rect);
-      return;
-    }
-
   hbi = _pango_cairo_font_private_get_hex_box_info (cf_priv);
   if (!hbi)
     {
@@ -735,20 +666,13 @@ _pango_cairo_font_private_get_glyph_extents_missing (PangoCairoFontPrivate *cf_p
       return;
     }
 
+  ch = glyph & ~PANGO_GLYPH_UNKNOWN_FLAG;
+
+  rows = hbi->rows;
   if (G_UNLIKELY (glyph == PANGO_GLYPH_INVALID_INPUT || ch > 0x10FFFF))
-    {
-      rows = hbi->rows;
-      cols = 1;
-    }
-  else if (pango_get_ignorable_size (ch, &rows, &cols))
-    {
-      /* We special-case ignorables when rendering hex boxes */
-    }
+    cols = 1;
   else
-    {
-      rows = hbi->rows;
-      cols = ((glyph & ~PANGO_GLYPH_UNKNOWN_FLAG) > 0xffff ? 6 : 4) / rows;
-    }
+    cols = ((glyph & ~PANGO_GLYPH_UNKNOWN_FLAG) > 0xffff ? 6 : 4) / rows;
 
   if (ink_rect)
     {
@@ -822,7 +746,6 @@ _pango_cairo_font_private_glyph_extents_cache_init (PangoCairoFontPrivate *cf_pr
   return TRUE;
 }
 
-
 /* Fills in the glyph extents cache entry
  */
 static void
